/* @TITLE "pattern.h: Define the access patterns"*/
/* 
 * The access patterns are currently defined as a run-time parameter in
 * driver.c, and then used to call the write function in cp.ca.
 * 
 * variables:
 *   Pattern
 *   PatternNames
 *
 * To add a (generic) pattern, you must modify
 *    pattern.h	    	enum
 *    pattern.c	    	PatternNames, PatternIsWrite
 *    cp.ca 	    	PatternFuncs
 *    iopfs-general.ca	switch statement, new mapping function
 * To add a really new pattern you need to change a lot more!
 *
 * Part of 
 *           The STARFISH Parallel file-system simulator
 *      (Simulation Tool for Advanced Research in File Systems)
 *
 *                              David Kotz
 *                          Dartmouth College
 *                             Version 2.0
 *                             January 1995
 *                         dfk@cs.dartmouth.edu
 */

/* $Id: pattern.h,v 2.0 95/01/18 12:17:58 dfk RELEASE $ */

#ifndef PATTERN_H
#define PATTERN_H

#include "dmcache.h"

/* Changing this requires changing NUMPATTERNS below,
 * and PatternNames[] in pattern.c 
 * and PatternIsWrite[] in pattern.c 
 * and PatternFuncs in cp.ca
 */
enum pattern_e {
    NULLCOMM_TEST,	      /* not a real pattern; a communications test */
    DATACOMM_TEST,	      /* not a real pattern; a communications test */
    MEMGET_TEST,	      /* not a real pattern; a Memget test */
    COPY_TEST,		      /* not a real pattern; a word-copy test */
    DISK_TEST,		      /* not a real pattern; a disk device test */

    LUINIT_PATTERN,	      /* special pattern; initialize for LU */
    LU_PATTERN,		      /* special pattern, LU decomposition */

    RECORD_READ_LW,
    RECORD_READ_LPS,
    BLOCK_READ_LW,
    RECORD_WRITE_LW,
    BLOCK_WRITE_LW,
    WHOLE_READ_LW,
    WHOLE_READ_LW1,
    WHOLE_WRITE_LW1,
    WHOLE_READ_SEG,
    WHOLE_WRITE_SEG,

    /* Read, 1-D distributions */
    WHOLE_READ_NONE,
    WHOLE_READ_BLOCK,
    WHOLE_READ_CYCLIC,

    /* Read, 2-D DISTRIBUTION(NONE, ...) */
    WHOLE_READ_NONE_NONE,
    WHOLE_READ_NONE_BLOCK,
    WHOLE_READ_NONE_CYCLIC,

    /* Read, 2-D DISTRIBUTION(BLOCK, ...) */
    WHOLE_READ_BLOCK_NONE,
    WHOLE_READ_BLOCK_BLOCK,
    WHOLE_READ_BLOCK_CYCLIC,

    /* Read, 2-D DISTRIBUTION(CYCLIC, ...) */
    WHOLE_READ_CYCLIC_NONE,
    WHOLE_READ_CYCLIC_BLOCK,
    WHOLE_READ_CYCLIC_CYCLIC,

    /* Write, 1-D distributions */
    WHOLE_WRITE_NONE,
    WHOLE_WRITE_BLOCK,
    WHOLE_WRITE_CYCLIC,

    /* Write, 2-D DISTRIBUTION(NONE, ...) */
    WHOLE_WRITE_NONE_NONE,
    WHOLE_WRITE_NONE_BLOCK,
    WHOLE_WRITE_NONE_CYCLIC,

    /* Write, 2-D DISTRIBUTION(BLOCK, ...) */
    WHOLE_WRITE_BLOCK_NONE,
    WHOLE_WRITE_BLOCK_BLOCK,
    WHOLE_WRITE_BLOCK_CYCLIC,

    /* Write, 2-D DISTRIBUTION(CYCLIC, ...) */
    WHOLE_WRITE_CYCLIC_NONE,
    WHOLE_WRITE_CYCLIC_BLOCK,
    WHOLE_WRITE_CYCLIC_CYCLIC

  /* don't forget to update NUMPATTERNS, below, pattern.c, and cp.ca */
};
#define NUMPATTERNS 41

typedef enum pattern_e PATTERN;

/* large enough for a string to hold a pattern name */
#define PATNAMELENGTH 15

/* GLOBAL variables, defined in pattern.c */
extern char PatternNames[NUMPATTERNS][PATNAMELENGTH];
extern boolean PatternIsWrite[NUMPATTERNS];

/* GLOBAL PARAMETER */
extern PATTERN Pattern;		/* access pattern to use */

/* Macros/functions for working with BLOCK and CYCLIC distributions.
 * nProcs is the number of processors involved in that dimension
 * (rows or cols).
 */
extern ulong NumRowsBLOCK(int CPnum, int nProcs, ulong nRows);

#define NumRowsCYCLIC(CPnum, nProcs, nRows) \
    ((nRows) / (nProcs) + ((CPnum) < (nRows) % (nProcs)))

#define NumColsBLOCK(CPnum, nProcs, nCols)  \
    ((nCols) / (nProcs))     /* PLUS you MUST check that nCols % nProcs == 0 */

#define NumColsCYCLIC(CPnum, nProcs, nCols) \
    ((nCols) / (nProcs) + ((CPnum) < (nCols) % (nProcs)))

#endif /* PATTERN_H */
